#ifndef LIB_LEXEME_H 
#define LIB_LEXEME_H

enum {
	CHARCLASS_EOF,
	CHARCLASS_UNKNOWN,
	CHARCLASS_LETTER,
	CHARCLASS_DIGIT,
	CHARCLASS_WHITESPACE
};

enum {
	TOKEN_EOF,
	TOKEN_UNKNOWN,
	TOKEN_NAME,
	TOKEN_CONST_INT,
	TOKEN_CONST_FLOAT,
	TOKEN_CONST_STRING,
	TOKEN_MUL,
	TOKEN_DIV,
	TOKEN_ADD,
	TOKEN_SUB,
	TOKEN_PAR_OPEN,
	TOKEN_PAR_CLOSE,
	TOKEN_COMMA,
	TOKEN_ASSIGN,
	TOKEN_SEMICOLON,
	TOKEN_QUESTION,
	TOKEN_COLON,
	TOKEN_BRACE_OPEN,
	TOKEN_BRACE_CLOSE,
	TOKEN_PERCENT,
	TOKEN_DOT,
	TOKEN_NOT_BOOL,
	TOKEN_NOT_INT,

	TOKEN_SHL,
	TOKEN_SHR,
	TOKEN_G,
	TOKEN_L,
	TOKEN_GE,
	TOKEN_LE,
	TOKEN_EQ,
	TOKEN_NE,
	TOKEN_AND_INT,
	TOKEN_OR_INT,
	TOKEN_XOR_INT,
	TOKEN_AND_BOOL,
	TOKEN_OR_BOOL
};

static const char *tokenName[] = {
	"TOKEN_EOF",
	"TOKEN_UNKNOWN",
	"TOKEN_NAME",
	"TOKEN_CONST_INT",
	"TOKEN_CONST_FLOAT",
	"TOKEN_CONST_STRING",
	"TOKEN_MUL",
	"TOKEN_DIV",
	"TOKEN_ADD",
	"TOKEN_SUB",
	"TOKEN_PAR_OPEN",
	"TOKEN_PAR_CLOSE",
	"TOKEN_COMMA",
	"TOKEN_ASSIGN",
	"TOKEN_SEMICOLON",
	"TOKEN_QUESTION",
	"TOKEN_COLON",
	"TOKEN_BRACE_OPEN",
	"TOKEN_BRACE_CLOSE",
	"TOKEN_PERCENT",
	"TOKEN_NOT_BOOL",
	"TOKEN_NOT_INT",

	"TOKEN_SHL",
	"TOKEN_SHR",
	"TOKEN_G",
	"TOKEN_L",
	"TOKEN_GE",
	"TOKEN_LE",
	"TOKEN_EQ",
	"TOKEN_NE",
	"TOKEN_AND_INT",
	"TOKEN_OR_INT",
	"TOKEN_XOR_INT",
	"TOKEN_AND_BOOL",
	"TOKEN_OR_BOOL",
};

#define LEXEME_DEPTH	2

class LexemeParser {
//instance vars:
protected:
	void clear();

	int charClass;
	char nextChar;

	char lexeme[LEXEME_DEPTH][256];
	int lastLexemeIndex;// = 0;
	int nextToken[LEXEME_DEPTH];	// = TOKEN_UNKNOWN;

	const char *readBuffer;
	int readBufferSize;
	int readBufferIndex;

public:
	int lineCount;
	int colCount;

//functions:
public:
	LexemeParser();
	LexemeParser(const char *src, int size);
	void setReadBuffer(const char *src, int size);
protected:

	void getChar();
	void addChar();
	int lookup(const char *lex);

public:
	//gets the next lexeme (similar to getChar's operation)
	//stores in nextToken
	void process();
	//returns the currently stored lexeme
	int token(int index = 0);
	//returns the name
	const char *lexemeString(int index = 0);
};


#endif
